@@ -6,6 +6,8 @@ Currently implemented:
 
   SimplePOP3Retriever
   SimplePOP3SSLRetriever
+  BrokenUIDLPOP3Retriever
+  BrokenUIDLPOP3SSLRetriever
   MultidropPOP3Retriever
   MultidropPOP3SSLRetriever
   MultidropSDPSRetriever
@@ -19,8 +21,9 @@ Currently implemented:
 
 __all__ = [
     'SimplePOP3Retriever',
-    'BrokenUIDLPOP3Retriever',
     'SimplePOP3SSLRetriever',
+    'BrokenUIDLPOP3Retriever',
+    'BrokenUIDLPOP3SSLRetriever',
     'MultidropPOP3Retriever',
     'MultidropPOP3SSLRetriever',
     'MultidropSDPSRetriever',
@@ -35,11 +38,12 @@ import poplib
 import imaplib
 import types
 
-from exceptions import *
-from constants import *
-from utilities import *
-from baseclasses import ConfigurableBase
-from _retrieverbases import *
+from getmailcore.exceptions import *
+from getmailcore.constants import *
+from getmailcore.utilities import *
+from getmailcore.baseclasses import *
+from getmailcore._retrieverbases import *
+
 
 #
 # Functional classes
@@ -50,15 +54,16 @@ class SimplePOP3Retriever(POP3RetrieverBase, POP3initMixIn):
     '''Retriever class for single-user POP3 mailboxes.
     '''
     _confitems = (
-        {'name' : 'configparser', 'type' : types.InstanceType, 'default' : None},
-        {'name' : 'getmaildir', 'type' : str, 'default' : '~/.getmail/'},
+        ConfInstance(name='configparser', required=False),
+        ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
-        {'name' : 'timeout', 'type' : int, 'default' : 180},
-        {'name' : 'server', 'type' : str},
-        {'name' : 'port', 'type' : int, 'default' : 110},
-        {'name' : 'username', 'type' : str},
-        {'name' : 'password', 'type' : str, 'default' : None},
-        {'name' : 'use_apop', 'type' : bool, 'default' : False},
+        ConfInt(name='timeout', required=False, default=180),
+        ConfString(name='server'),
+        ConfInt(name='port', required=False, default=110),
+        ConfString(name='username'),
+        ConfPassword(name='password', required=False, default=None),
+        ConfBool(name='use_apop', required=False, default=False),
+        ConfBool(name='delete_dup_msgids', required=False, default=False),
     )
     received_from = None
     received_with = 'POP3'
@@ -74,85 +79,26 @@ class SimplePOP3Retriever(POP3RetrieverBase, POP3initMixIn):
 
     def showconf(self):
         self.log.trace()
-        self.log.info('SimplePOP3Retriever(%s)' % self._confstring() 
-            + os.linesep)
-
-#######################################
-class BrokenUIDLPOP3Retriever(POP3RetrieverBase, POP3initMixIn):
-    '''Retriever class for single-user POP3 mailboxes on servers that do not
-    properly assign unique IDs to messages.  Since with these broken servers
-    we cannot rely on UIDL, we have to use message numbers, which are unique
-    within a POP3 session, but which change across sessions.  This class
-    therefore can not be used to leave old mail on the server and download
-    only new mail.
-    '''
-    _confitems = (
-        {'name' : 'configparser', 'type' : types.InstanceType, 'default' : None},
-        {'name' : 'getmaildir', 'type' : str, 'default' : '~/.getmail/'},
-
-        {'name' : 'timeout', 'type' : int, 'default' : 180},
-        {'name' : 'server', 'type' : str},
-        {'name' : 'port', 'type' : int, 'default' : 110},
-        {'name' : 'username', 'type' : str},
-        {'name' : 'password', 'type' : str, 'default' : None},
-        {'name' : 'use_apop', 'type' : bool, 'default' : False},
-    )
-    received_from = None
-    received_with = 'POP3'
-    received_by = localhostname()
-
-    def _read_oldmailfile(self):
-        '''Force list of old messages to be empty by making this a no-op, so
-        duplicated IDs are always treated as new messages.'''
-        self.log.trace()
-
-    def write_oldmailfile(self):
-        '''Short-circuit writing the oldmail file.'''
-        self.log.trace()
-
-    def _getmsglist(self):
-        '''Don't rely on UIDL; instead, use just the message number.'''
-        self.log.trace()
-        try:
-            response, msglist, octets = self.conn.list()
-            for line in msglist:
-                msgnum = int(line.split()[0])
-                msgsize = int(line.split()[1])
-                self.msgids.append(msgnum)
-                self.msgsizes[msgnum] = msgsize
-        except poplib.error_proto, o:
-            raise getmailOperationError('POP error (%s)' % o)
-        self.gotmsglist = True
-
-    def __str__(self):
-        self.log.trace()
-        return 'BrokenUIDLPOP3Retriever:%s@%s:%s' % (
-            self.conf.get('username', 'username'),
-            self.conf.get('server', 'server'),
-            self.conf.get('port', 'port')
-        )
-
-    def showconf(self):
-        self.log.trace()
-        self.log.info('BrokenUIDLPOP3Retriever(%s)' % self._confstring()
-            + os.linesep)
+        self.log.info('SimplePOP3Retriever(%s)' % self._confstring()
+                      + os.linesep)
 
 #######################################
 class SimplePOP3SSLRetriever(POP3RetrieverBase, POP3SSLinitMixIn):
     '''Retriever class for single-user POP3-over-SSL mailboxes.
     '''
     _confitems = (
-        {'name' : 'configparser', 'type' : types.InstanceType, 'default' : None},
-        {'name' : 'getmaildir', 'type' : str, 'default' : '~/.getmail/'},
+        ConfInstance(name='configparser', required=False),
+        ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
-        {'name' : 'timeout', 'type' : int, 'default' : 180},
-        {'name' : 'server', 'type' : str},
-        {'name' : 'port', 'type' : int, 'default' : POP3_ssl_port},
-        {'name' : 'username', 'type' : str},
-        {'name' : 'password', 'type' : str, 'default' : None},
-        {'name' : 'use_apop', 'type' : bool, 'default' : False},
-        {'name' : 'keyfile', 'type' : str, 'default' : None},
-        {'name' : 'certfile', 'type' : str, 'default' : None},
+        ConfInt(name='timeout', required=False, default=180),
+        ConfString(name='server'),
+        ConfInt(name='port', required=False, default=POP3_ssl_port),
+        ConfString(name='username'),
+        ConfPassword(name='password', required=False, default=None),
+        ConfBool(name='use_apop', required=False, default=False),
+        ConfBool(name='delete_dup_msgids', required=False, default=False),
+        ConfFile(name='keyfile', required=False, default=None),
+        ConfFile(name='certfile', required=False, default=None),
     )
     received_from = None
     received_with = 'POP3-SSL'
@@ -169,23 +115,122 @@ class SimplePOP3SSLRetriever(POP3RetrieverBase, POP3SSLinitMixIn):
     def showconf(self):
         self.log.trace()
         self.log.info('SimplePOP3SSLRetriever(%s)' % self._confstring()
-            + os.linesep)
+                      + os.linesep)
+
+#######################################
+class BrokenUIDLPOP3RetrieverBase(POP3RetrieverBase):
+    '''Retriever base class for single-user POP3 mailboxes on servers that do
+    not properly assign unique IDs to messages.  Since with these broken servers
+    we cannot rely on UIDL, we have to use message numbers, which are unique
+    within a POP3 session, but which change across sessions.  This class
+    therefore can not be used to leave old mail on the server and download only
+    new mail.
+    '''
+    received_from = None
+    received_by = localhostname()
+
+    def _read_oldmailfile(self):
+        '''Force list of old messages to be empty by making this a no-op, so
+        duplicated IDs are always treated as new messages.'''
+        self.log.trace()
+
+    def write_oldmailfile(self, **kwargs):
+        '''Short-circuit writing the oldmail file.'''
+        self.log.trace()
+
+    def _getmsglist(self):
+        '''Don't rely on UIDL; instead, use just the message number.'''
+        self.log.trace()
+        try:
+            (response, msglist, octets) = self.conn.list()
+            for line in msglist:
+                msgnum = int(line.split()[0])
+                msgsize = int(line.split()[1])
+                self.msgnum_by_msgid[msgnum] = msgnum
+                self.msgid_by_msgnum[msgnum] = msgnum
+                self.msgsizes[msgnum] = msgsize
+            self.sorted_msgnum_msgid = sorted(self.msgid_by_msgnum.items())
+        except poplib.error_proto, o:
+            raise getmailOperationError('POP error (%s)' % o)
+        self.gotmsglist = True
+
+#######################################
+class BrokenUIDLPOP3Retriever(BrokenUIDLPOP3RetrieverBase, POP3initMixIn):
+    '''For broken POP3 servers without SSL.
+    '''
+    _confitems = (
+        ConfInstance(name='configparser', required=False),
+        ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
+
+        ConfInt(name='timeout', required=False, default=180),
+        ConfString(name='server'),
+        ConfInt(name='port', required=False, default=110),
+        ConfString(name='username'),
+        ConfPassword(name='password', required=False, default=None),
+        ConfBool(name='use_apop', required=False, default=False),
+    )
+    received_with = 'POP3'
+
+    def __str__(self):
+        self.log.trace()
+        return 'BrokenUIDLPOP3Retriever:%s@%s:%s' % (
+            self.conf.get('username', 'username'),
+            self.conf.get('server', 'server'),
+            self.conf.get('port', 'port')
+        )
+
+    def showconf(self):
+        self.log.trace()
+        self.log.info('BrokenUIDLPOP3Retriever(%s)' % self._confstring()
+                      + os.linesep)
+
+#######################################
+class BrokenUIDLPOP3SSLRetriever(BrokenUIDLPOP3RetrieverBase, POP3SSLinitMixIn):
+    '''For broken POP3 servers with SSL.
+    '''
+    _confitems = (
+        ConfInstance(name='configparser', required=False),
+        ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
+
+        ConfInt(name='timeout', required=False, default=180),
+        ConfString(name='server'),
+        ConfInt(name='port', required=False, default=POP3_ssl_port),
+        ConfString(name='username'),
+        ConfPassword(name='password', required=False, default=None),
+        ConfBool(name='use_apop', required=False, default=False),
+        ConfFile(name='keyfile', required=False, default=None),
+        ConfFile(name='certfile', required=False, default=None),
+    )
+    received_with = 'POP3-SSL'
+
+    def __str__(self):
+        self.log.trace()
+        return 'BrokenUIDLPOP3SSLRetriever:%s@%s:%s' % (
+            self.conf.get('username', 'username'),
+            self.conf.get('server', 'server'),
+            self.conf.get('port', 'port')
+        )
+
+    def showconf(self):
+        self.log.trace()
+        self.log.info('BrokenUIDLPOP3SSLRetriever(%s)' % self._confstring()
+                      + os.linesep)
 
 #######################################
 class MultidropPOP3Retriever(MultidropPOP3RetrieverBase, POP3initMixIn):
     '''Retriever class for multi-drop POP3 mailboxes.
     '''
     _confitems = (
-        {'name' : 'configparser', 'type' : types.InstanceType, 'default' : None},
-        {'name' : 'getmaildir', 'type' : str, 'default' : '~/.getmail/'},
+        ConfInstance(name='configparser', required=False),
+        ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
-        {'name' : 'timeout', 'type' : int, 'default' : 180},
-        {'name' : 'server', 'type' : str},
-        {'name' : 'port', 'type' : int, 'default' : 110},
-        {'name' : 'username', 'type' : str},
-        {'name' : 'password', 'type' : str, 'default' : None},
-        {'name' : 'use_apop', 'type' : bool, 'default' : False},
-        {'name' : 'envelope_recipient', 'type' : str},
+        ConfInt(name='timeout', required=False, default=180),
+        ConfString(name='server'),
+        ConfInt(name='port', required=False, default=110),
+        ConfString(name='username'),
+        ConfPassword(name='password', required=False, default=None),
+        ConfBool(name='use_apop', required=False, default=False),
+        ConfString(name='envelope_recipient'),
     )
     received_from = None
     received_with = 'POP3'
@@ -202,25 +247,25 @@ class MultidropPOP3Retriever(MultidropPOP3RetrieverBase, POP3initMixIn):
     def showconf(self):
         self.log.trace()
         self.log.info('MultidropPOP3Retriever(%s)' % self._confstring()
-            + os.linesep)
+                      + os.linesep)
 
 #######################################
 class MultidropPOP3SSLRetriever(MultidropPOP3RetrieverBase, POP3SSLinitMixIn):
     '''Retriever class for multi-drop POP3-over-SSL mailboxes.
     '''
     _confitems = (
-        {'name' : 'configparser', 'type' : types.InstanceType, 'default' : None},
-        {'name' : 'getmaildir', 'type' : str, 'default' : '~/.getmail/'},
+        ConfInstance(name='configparser', required=False),
+        ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
-        {'name' : 'timeout', 'type' : int, 'default' : 180},
-        {'name' : 'server', 'type' : str},
-        {'name' : 'port', 'type' : int, 'default' : POP3_ssl_port},
-        {'name' : 'username', 'type' : str},
-        {'name' : 'password', 'type' : str, 'default' : None},
-        {'name' : 'use_apop', 'type' : bool, 'default' : False},
-        {'name' : 'envelope_recipient', 'type' : str},
-        {'name' : 'keyfile', 'type' : str, 'default' : None},
-        {'name' : 'certfile', 'type' : str, 'default' : None},
+        ConfInt(name='timeout', required=False, default=180),
+        ConfString(name='server'),
+        ConfInt(name='port', required=False, default=POP3_ssl_port),
+        ConfString(name='username'),
+        ConfPassword(name='password', required=False, default=None),
+        ConfBool(name='use_apop', required=False, default=False),
+        ConfString(name='envelope_recipient'),
+        ConfFile(name='keyfile', required=False, default=None),
+        ConfFile(name='certfile', required=False, default=None),
     )
     received_from = None
     received_with = 'POP3-SSL'
@@ -237,7 +282,7 @@ class MultidropPOP3SSLRetriever(MultidropPOP3RetrieverBase, POP3SSLinitMixIn):
     def showconf(self):
         self.log.trace()
         self.log.info('MultidropPOP3SSLRetriever(%s)' % self._confstring()
-            + os.linesep)
+                      + os.linesep)
 
 #######################################
 class MultidropSDPSRetriever(SimplePOP3Retriever, POP3initMixIn):
@@ -251,16 +296,16 @@ class MultidropSDPSRetriever(SimplePOP3Retriever, POP3initMixIn):
     Support originally requested by Paul Clifford for getmail v.2/3.
     '''
     _confitems = (
-        {'name' : 'configparser', 'type' : types.InstanceType, 'default' : None},
-        {'name' : 'getmaildir', 'type' : str, 'default' : '~/.getmail/'},
+        ConfInstance(name='configparser', required=False),
+        ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
-        {'name' : 'timeout', 'type' : int, 'default' : 180},
-        {'name' : 'server', 'type' : str},
-        {'name' : 'port', 'type' : int, 'default' : 110},
-        {'name' : 'username', 'type' : str},
-        {'name' : 'password', 'type' : str, 'default' : None},
+        ConfInt(name='timeout', required=False, default=180),
+        ConfString(name='server'),
+        ConfInt(name='port', required=False, default=110),
+        ConfString(name='username'),
+        ConfPassword(name='password', required=False, default=None),
         # Demon apparently doesn't support APOP
-        {'name' : 'use_apop', 'type' : bool, 'default' : False},
+        ConfBool(name='use_apop', required=False, default=False),
     )
 
     received_from = None
@@ -278,7 +323,7 @@ class MultidropSDPSRetriever(SimplePOP3Retriever, POP3initMixIn):
     def showconf(self):
         self.log.trace()
         self.log.info('MultidropSDPSRetriever(%s)' % self._confstring()
-            + os.linesep)
+                      + os.linesep)
 
     def _getmsgbyid(self, msgid):
         self.log.trace()
@@ -289,8 +334,9 @@ class MultidropSDPSRetriever(SimplePOP3Retriever, POP3initMixIn):
             msgnum = self._getmsgnumbyid(msgid)
             resp, lines, octets = self.conn._longcmd('*ENV %i' % msgnum)
         except poplib.error_proto, o:
-            raise getmailConfigurationError('server does not support *ENV (%s)'
-                % o)
+            raise getmailConfigurationError(
+                'server does not support *ENV (%s)' % o
+            )
         if len(lines) < 4:
             raise getmailOperationError('short *ENV response (%s)' % lines)
         msg.sender = lines[2]
@@ -302,21 +348,22 @@ class SimpleIMAPRetriever(IMAPRetrieverBase, IMAPinitMixIn):
     '''Retriever class for single-user IMAPv4 mailboxes.
     '''
     _confitems = (
-        {'name' : 'configparser', 'type' : types.InstanceType, 'default' : None},
-        {'name' : 'getmaildir', 'type' : str, 'default' : '~/.getmail/'},
-
-        {'name' : 'timeout', 'type' : int, 'default' : 180},
-        {'name' : 'server', 'type' : str},
-        {'name' : 'port', 'type' : int, 'default' : imaplib.IMAP4_PORT},
-        {'name' : 'username', 'type' : str},
-        {'name' : 'password', 'type' : str, 'default' : None},
-
-        {'name' : 'mailboxes', 'type' : tuple, 'default' : ('INBOX', )},
-        {'name' : 'move_on_delete', 'type' : str, 'default' : None},
+        ConfInstance(name='configparser', required=False),
+        ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
+        ConfInt(name='timeout', required=False, default=180),
+        ConfString(name='server'),
+        ConfInt(name='port', required=False, default=imaplib.IMAP4_PORT),
+        ConfString(name='username'),
+        ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='mailboxes', required=False,
+                           default="('INBOX', )"),
+        ConfBool(name='use_peek', required=False, default=True),
+        ConfString(name='move_on_delete', required=False, default=None),
         # imaplib.IMAP4.login_cram_md5() requires the (unimplemented)
-        # .authenticate(), so we can't do this yet.
-        {'name' : 'use_cram_md5', 'type' : bool, 'default' : False},
+        # .authenticate(), so we can't do this yet (?).
+        ConfBool(name='use_cram_md5', required=False, default=False),
+        ConfBool(name='use_kerberos', required=False, default=False),
     )
     received_from = None
     received_with = 'IMAP4'
@@ -333,32 +380,32 @@ class SimpleIMAPRetriever(IMAPRetrieverBase, IMAPinitMixIn):
     def showconf(self):
         self.log.trace()
         self.log.info('SimpleIMAPRetriever(%s)' % self._confstring()
-            + os.linesep)
+                      + os.linesep)
 
 #######################################
 class SimpleIMAPSSLRetriever(IMAPRetrieverBase, IMAPSSLinitMixIn):
     '''Retriever class for single-user IMAPv4-over-SSL mailboxes.
     '''
     _confitems = (
-        {'name' : 'configparser', 'type' : types.InstanceType, 'default' : None},
-        {'name' : 'getmaildir', 'type' : str, 'default' : '~/.getmail/'},
+        ConfInstance(name='configparser', required=False),
+        ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
         # socket.ssl() and socket timeouts are incompatible in Python 2.3
-        #{'name' : 'timeout', 'type' : int, 'default' : 180},
-        {'name' : 'server', 'type' : str},
-        {'name' : 'port', 'type' : int, 'default' : imaplib.IMAP4_SSL_PORT},
-        {'name' : 'username', 'type' : str},
-        {'name' : 'password', 'type' : str, 'default' : None},
-
-        {'name' : 'mailboxes', 'type' : tuple, 'default' : ('INBOX', )},
-        {'name' : 'move_on_delete', 'type' : str, 'default' : None},
-
-        {'name' : 'keyfile', 'type' : str, 'default' : None},
-        {'name' : 'certfile', 'type' : str, 'default' : None},
-
+        #ConfInt(name='timeout', required=False, default=180),
+        ConfString(name='server'),
+        ConfInt(name='port', required=False, default=imaplib.IMAP4_SSL_PORT),
+        ConfString(name='username'),
+        ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='mailboxes', required=False,
+                           default="('INBOX', )"),
+        ConfBool(name='use_peek', required=False, default=True),
+        ConfString(name='move_on_delete', required=False, default=None),
+        ConfFile(name='keyfile', required=False, default=None),
+        ConfFile(name='certfile', required=False, default=None),
         # imaplib.IMAP4.login_cram_md5() requires the (unimplemented)
-        # .authenticate(), so we can't do this yet.
-        {'name' : 'use_cram_md5', 'type' : bool, 'default' : False},
+        # .authenticate(), so we can't do this yet (?).
+        ConfBool(name='use_cram_md5', required=False, default=False),
+        ConfBool(name='use_kerberos', required=False, default=False),
     )
     received_from = None
     received_with = 'IMAP4-SSL'
@@ -375,30 +422,30 @@ class SimpleIMAPSSLRetriever(IMAPRetrieverBase, IMAPSSLinitMixIn):
     def showconf(self):
         self.log.trace()
         self.log.info('SimpleIMAPSSLRetriever(%s)' % self._confstring()
-            + os.linesep)
+                      + os.linesep)
 
 #######################################
 class MultidropIMAPRetriever(MultidropIMAPRetrieverBase, IMAPinitMixIn):
     '''Retriever class for multi-drop IMAPv4 mailboxes.
     '''
     _confitems = (
-        {'name' : 'configparser', 'type' : types.InstanceType, 'default' : None},
-        {'name' : 'getmaildir', 'type' : str, 'default' : '~/.getmail/'},
-
-        {'name' : 'timeout', 'type' : int, 'default' : 180},
-        {'name' : 'server', 'type' : str},
-        {'name' : 'port', 'type' : int, 'default' : imaplib.IMAP4_PORT},
-        {'name' : 'username', 'type' : str},
-        {'name' : 'password', 'type' : str, 'default' : None},
-
-        {'name' : 'mailboxes', 'type' : tuple, 'default' : ('INBOX', )},
-        {'name' : 'move_on_delete', 'type' : str, 'default' : None},
+        ConfInstance(name='configparser', required=False),
+        ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
+        ConfInt(name='timeout', required=False, default=180),
+        ConfString(name='server'),
+        ConfInt(name='port', required=False, default=imaplib.IMAP4_PORT),
+        ConfString(name='username'),
+        ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='mailboxes', required=False,
+                           default="('INBOX', )"),
+        ConfBool(name='use_peek', required=False, default=True),
+        ConfString(name='move_on_delete', required=False, default=None),
         # imaplib.IMAP4.login_cram_md5() requires the (unimplemented)
-        # .authenticate(), so we can't do this yet.
-        {'name' : 'use_cram_md5', 'type' : bool, 'default' : False},
-
-        {'name' : 'envelope_recipient', 'type' : str},
+        # .authenticate(), so we can't do this yet (?).
+        ConfBool(name='use_cram_md5', required=False, default=False),
+        ConfBool(name='use_kerberos', required=False, default=False),
+        ConfString(name='envelope_recipient'),
     )
     received_from = None
     received_with = 'IMAP4'
@@ -415,34 +462,33 @@ class MultidropIMAPRetriever(MultidropIMAPRetrieverBase, IMAPinitMixIn):
     def showconf(self):
         self.log.trace()
         self.log.info('MultidropIMAPRetriever(%s)' % self._confstring()
-            + os.linesep)
+                      + os.linesep)
 
 #######################################
 class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
     '''Retriever class for multi-drop IMAPv4-over-SSL mailboxes.
     '''
     _confitems = (
-        {'name' : 'configparser', 'type' : types.InstanceType, 'default' : None},
-        {'name' : 'getmaildir', 'type' : str, 'default' : '~/.getmail/'},
+        ConfInstance(name='configparser', required=False),
+        ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
         # socket.ssl() and socket timeouts are incompatible in Python 2.3
-        #{'name' : 'timeout', 'type' : int, 'default' : 180},
-        {'name' : 'server', 'type' : str},
-        {'name' : 'port', 'type' : int, 'default' : imaplib.IMAP4_SSL_PORT},
-        {'name' : 'username', 'type' : str},
-        {'name' : 'password', 'type' : str, 'default' : None},
-
-        {'name' : 'mailboxes', 'type' : tuple, 'default' : ('INBOX', )},
-        {'name' : 'move_on_delete', 'type' : str, 'default' : None},
-
-        {'name' : 'keyfile', 'type' : str, 'default' : None},
-        {'name' : 'certfile', 'type' : str, 'default' : None},
-
+        #ConfInt(name='timeout', required=False, default=180),
+        ConfString(name='server'),
+        ConfInt(name='port', required=False, default=imaplib.IMAP4_SSL_PORT),
+        ConfString(name='username'),
+        ConfPassword(name='password', required=False, default=None),
+        ConfTupleOfStrings(name='mailboxes', required=False,
+                           default="('INBOX', )"),
+        ConfBool(name='use_peek', required=False, default=True),
+        ConfString(name='move_on_delete', required=False, default=None),
+        ConfFile(name='keyfile', required=False, default=None),
+        ConfFile(name='certfile', required=False, default=None),
         # imaplib.IMAP4.login_cram_md5() requires the (unimplemented)
-        # .authenticate(), so we can't do this yet.
-        {'name' : 'use_cram_md5', 'type' : bool, 'default' : False},
-
-        {'name' : 'envelope_recipient', 'type' : str},
+        # .authenticate(), so we can't do this yet (?).
+        ConfBool(name='use_cram_md5', required=False, default=False),
+        ConfBool(name='use_kerberos', required=False, default=False),
+        ConfString(name='envelope_recipient'),
     )
     received_from = None
     received_with = 'IMAP4-SSL'
@@ -459,4 +505,4 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
     def showconf(self):
         self.log.trace()
         self.log.info('MultidropIMAPSSLRetriever(%s)' % self._confstring()
-            + os.linesep)
+                      + os.linesep)
